/*
  CTextureJPG - uses libjpeg
  Created: 20/05/1999
  Author : Sylvain Lefebvre
*/
//---------------------------------------------------------------------------
#include "CTextureJPG.h"
#include "CLibTextureException.h"
//---------------------------------------------------------------------------
/*
  LibJPEG stuff
 */
//---------------------------------------------------------------------------
#include <iostream>
#include <cstdio>
#include <cstring>
#ifndef WIN32
#  ifdef __cplusplus
   extern "C" {
#  endif
#endif
#include "jpeglib.h"
#ifndef WIN32
#  ifdef __cplusplus
   }
#  endif
#endif
#include <setjmp.h>

using namespace std;

//---------------------------------------------------------------------------

CTextureJPG::CTextureJPG(const char *n) : CTexture()
{
  strcpy(m_szName,n);
  CTextureJPG::load();
}
//---------------------------------------------------------------------------
void CTextureJPG::load()
{
  /* on charge le JPG */
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  FILE * infile;		/* source file */
  JSAMPARRAY buffer;		/* Output row buffer */
  int row_stride;		/* physical row width in output buffer */
  int i,line=0;
  unsigned char *pbmp;

  m_iWidth=0;
  m_iHeight=0;
  m_Data=NULL;
  
  if ((infile = fopen(m_szName, "rb")) == NULL)
    throw CLibTextureException("Unable to load %s",m_szName);
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  jpeg_stdio_src(&cinfo, infile);
  (void) jpeg_read_header(&cinfo, TRUE);
  (void) jpeg_start_decompress(&cinfo);

  m_iWidth=cinfo.output_width;
  m_iHeight=cinfo.output_height;
  std::cerr << "Loading JPG image " << m_szName << " " << m_iWidth << "x" << m_iHeight 
       << std::endl;
  row_stride = cinfo.output_width * cinfo.output_components;
  m_iStride=row_stride;
  buffer = (*cinfo.mem->alloc_sarray)
  		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);
  m_Data=new unsigned char[cinfo.output_height*row_stride];
  line=0;
  while (cinfo.output_scanline < cinfo.output_height)
  {
    (void) jpeg_read_scanlines(&cinfo, buffer, 1);
    pbmp = (unsigned char *)(m_Data+row_stride*line);
    line++;
    for (i=0;i<row_stride/3;i++)
    {
      pbmp[i*3] = buffer[0][i*3];
      pbmp[i*3+1] = buffer[0][i*3+1];
      pbmp[i*3+2] = buffer[0][i*3+2];
    }
  }
  (void) jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  fclose(infile);
}

//---------------------------------------------------------------------------

void CTextureJPG::save()
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;

  FILE * outfile;
  JSAMPROW row_pointer[1];
  int row_stride;

  // open file
  outfile=fopen(m_szName,"wb");
  if (NULL == outfile)
    return;
  cerr << "Saving JPG image " << m_szName << " " << m_iWidth << "x" << m_iHeight;

  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  jpeg_stdio_dest(&cinfo, outfile);

  cinfo.image_width = m_iWidth;
  cinfo.image_height = m_iHeight;
  cinfo.input_components = 3;
  cinfo.in_color_space = JCS_RGB;

  jpeg_set_defaults(&cinfo);
  jpeg_set_quality(&cinfo, 100, TRUE);

  jpeg_start_compress(&cinfo, TRUE);
  row_stride = m_iWidth * (isAlpha()?4:3);

  while (cinfo.next_scanline < cinfo.image_height) 
  {
    row_pointer[0] = & m_Data[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  jpeg_finish_compress(&cinfo);
  
  fclose(outfile);

  jpeg_destroy_compress(&cinfo);

  cerr << " ... done" << endl;
}

//---------------------------------------------------------------------------
