/*
  TGA Texture
  Created: 11/09/2000
  Author : Sylvain Lefebvre
*/
//---------------------------------------------------------------------------
#include "CTextureTGA.h"
#include "CLibTextureException.h"
//---------------------------------------------------------------------------
#include <iostream>
#include <stdio.h>
#include <string.h>
//---------------------------------------------------------------------------
using namespace std;
//---------------------------------------------------------------------------
CTextureTGA::CTextureTGA(const char *n) : CTexture()
{
  strcpy(m_szName,n);
  CTextureTGA::load();
}
//---------------------------------------------------------------------------
void CTextureTGA::load()
{
  FILE		*f;
  tga_header	h;
  unsigned char	c;
  int		i,j,t;

  m_iWidth=0;
  m_iHeight=0;
  m_Data=NULL;
  // open file
  f=fopen(m_szName,"rb");
  if (f == NULL)
    throw CLibTextureException("Unable to load tga %s",m_szName);
  // read header
  fread(&h.id_len,sizeof(h.id_len),1,f);
  fread(&h.cmap_type,sizeof(h.cmap_type),1,f);
  fread(&h.img_type,sizeof(h.img_type),1,f);
  fread(&h.cmap_origin,sizeof(h.cmap_origin),1,f);
  fread(&h.cmap_len,sizeof(h.cmap_len),1,f);
  fread(&h.cmap_entry_size,sizeof(h.cmap_entry_size),1,f);
  fread(&h.x,sizeof(h.x),1,f);
  fread(&h.y,sizeof(h.y),1,f);
  fread(&h.w,sizeof(h.w),1,f);
  fread(&h.h,sizeof(h.h),1,f);
  fread(&h.pix_size,sizeof(h.pix_size),1,f);
  fread(&h.img_desc,sizeof(h.img_desc),1,f);
  // check if valid
  if (h.img_type != 2 || h.cmap_type !=0 || (h.pix_size != 24 && h.pix_size != 32))
    throw CLibTextureException("Unable to load tga %s (bad file format)",m_szName);
  // jump over id
  fseek(f,h.id_len,SEEK_CUR);
  // alloc struct
  m_iWidth=h.w;
  m_iHeight=h.h;
  cerr << "Loading TGA image " << m_szName << " " << m_iWidth << "x" << m_iHeight 
       << (h.pix_size == 32 ? " RGBA" : " RGB" )
       << endl;
  if (h.pix_size == 32)
  {
    m_bAlpha=true;
    m_Data=new unsigned char[m_iWidth*m_iHeight*4];
    t=4;
    // read RGBA data
    fread(m_Data,m_iWidth*m_iHeight*4,1,f);
  }
  else
  {
    m_bAlpha=false;
    m_Data=new unsigned char[m_iWidth*m_iHeight*3];
    t=3;
    // read RGB data
    fread(m_Data,m_iWidth*m_iHeight*3,1,f);
  }
  // BGR -> RGB
  for (i=0;i<m_iWidth;i++)
  {
    for (j=0;j<m_iHeight;j++)
    {
      c=m_Data[(i+j*m_iWidth)*t];
      m_Data[(i+j*m_iWidth)*t]=m_Data[(i+j*m_iWidth)*t+2];
      m_Data[(i+j*m_iWidth)*t+2]=c;
    }
  }
  // flip ?
  if (h.img_desc & 32) // 0010 0000
  {
    flipX();
  }
  if (!(h.img_desc & 16)) // 0001 0000
  {
    flipY();
  }
  // close file
  fclose(f);
}
//---------------------------------------------------------------------------
void CTextureTGA::save()
{
  int            w,h,t;
  FILE		*f;
  tga_header	 header;

  w=m_iWidth;
  h=m_iHeight;
  t=(isAlpha()?4:3);
  // open file
  f=fopen(m_szName,"wb");
  if (NULL == f)
    return;
  cerr << "Saving TGA image " << m_szName << " " << w << "x" << h;
  // header
  memset(&header,0,sizeof(header));
  header.cmap_type=0;
  if (isAlpha())
    header.pix_size=32;
  else
    header.pix_size=24;
  header.img_type=2;
  header.img_desc=16;
  header.w=w;
  header.h=h;
  // write header
  fwrite(&header.id_len,sizeof(header.id_len),1,f);
  fwrite(&header.cmap_type,sizeof(header.cmap_type),1,f);
  fwrite(&header.img_type,sizeof(header.img_type),1,f);
  fwrite(&header.cmap_origin,sizeof(header.cmap_origin),1,f);
  fwrite(&header.cmap_len,sizeof(header.cmap_len),1,f);
  fwrite(&header.cmap_entry_size,sizeof(header.cmap_entry_size),1,f);
  fwrite(&header.x,sizeof(header.x),1,f);
  fwrite(&header.y,sizeof(header.y),1,f);
  fwrite(&header.w,sizeof(header.w),1,f);
  fwrite(&header.h,sizeof(header.h),1,f);
  fwrite(&header.pix_size,sizeof(header.pix_size),1,f);
  fwrite(&header.img_desc,sizeof(header.img_desc),1,f);
  // RGB -> BGR
  int           i,j;
  unsigned char c;
  for (i=0;i<w;i++)
  {
    for (j=0;j<h;j++)
    {
      c=m_Data[(i+j*w)*t];
      m_Data[(i+j*w)*t]=m_Data[(i+j*w)*t+2];
      m_Data[(i+j*w)*t+2]=c;
    }
  }
  // write data
  fwrite(m_Data,w*h*t,1,f);
  // BGR -> RGB
  for (i=0;i<w;i++)
  {
    for (j=0;j<h;j++)
    {
      c=m_Data[(i+j*w)*t];
      m_Data[(i+j*w)*t]=m_Data[(i+j*w)*t+2];
      m_Data[(i+j*w)*t+2]=c;
    }
  }
  // close file
  fclose(f);
  cerr << " ... done" << endl;
}
//---------------------------------------------------------------------------
