// ----------------------------------------------------------------------------
// CGPUmesh_GL_funcs.h
//
// The goal of this template class is to build a template mechanism
// above the OpenGL function pointers.
// The run time overhead is null as everything is resolved at compile time.
//
// Exemple: GLFuncs<float>::getFct(0)(4,ptr);
// => this calls glVertexAttrib1fvARB(4,ptr)
//
//                                                    (c) Sylvain Lefebvre 2003
// ----------------------------------------------------------------------------
#ifndef __GPUMESH_GL_FUNCS__
#define __GPUMESH_GL_FUNCS__

#include <GL/gl.h>

#ifndef GPUMESH_NO_GLUX
#  include <glux.h>
GLUX_LOAD(GL_ARB_multitexture)
GLUX_LOAD(GL_ARB_vertex_program)
#endif

// -----------------------------------------
namespace GPUmesh {
// -----------------------------------------

template <class T> struct GLFuncs;

template <> struct GLFuncs<MVF_null>
{
  typedef void (APIENTRY *func)(const MVF_null *);
  typedef void (APIENTRY *attfunc)(GLuint,const MVF_null *);
  typedef void (APIENTRY *texfunc)(GLenum,const MVF_null *);

  static GLFuncs<MVF_null>::func    getColor0Fct(int i) {return (NULL);}
  static GLFuncs<MVF_null>::attfunc getAttributeFct(int i) {return (NULL);}
  static GLFuncs<MVF_null>::func    getVertexFct(int i) {return (NULL);}
  static GLFuncs<MVF_null>::func    getNormalFct(int i) {return (NULL);}
  static GLFuncs<MVF_null>::texfunc getTexCoordnFct(int i) {return (NULL);}
};

// -----------------------------------------

template <> struct GLFuncs<float>
{
public:

  typedef void (APIENTRY *func)(const GLfloat *);
  typedef void (APIENTRY *attfunc)(GLuint,const GLfloat *);
  typedef void (APIENTRY *texfunc)(GLenum,const GLfloat *);

  static func getColor0Fct(int i)
  {
    static const func funcs[4]={NULL, NULL, glColor3fv, glColor4fv}; 
    return (funcs[i]);
  }
  static attfunc getAttributeFct(int i)
  {
    static const attfunc funcs[4]={glVertexAttrib1fvARB, glVertexAttrib2fvARB, glVertexAttrib3fvARB, glVertexAttrib4fvARB}; 
    return (funcs[i]);
  }
  static func getVertexFct(int i)
  {
    static const func funcs[4]={NULL, glVertex2fv, glVertex3fv, glVertex4fv}; 
    return (funcs[i]);
  }
  static func getNormalFct(int i)
  {
    static const func funcs[4]={NULL, NULL, glNormal3fv, NULL}; 
    return (funcs[i]);
  }
  static texfunc getTexCoordnFct(int i)
  {
    static const texfunc funcs[4]={glMultiTexCoord1fvARB, glMultiTexCoord2fvARB, glMultiTexCoord3fvARB, NULL}; 
    return (funcs[i]);
  }
  
};

// -----------------------------------------

template <> struct GLFuncs<double>
{
public:

  typedef void (APIENTRY *func)(const GLdouble *);
  typedef void (APIENTRY *attfunc)(GLuint,const GLdouble *);
  typedef void (APIENTRY *texfunc)(GLenum,const GLdouble *);

  static func getColor0Fct(int i)
  {
    static const func funcs[4]={NULL, NULL, glColor3dv, glColor4dv}; 
    return (funcs[i]);
  }
  static attfunc getAttributeFct(int i)
  {
    static const attfunc funcs[4]={glVertexAttrib1dvARB, glVertexAttrib2dvARB, glVertexAttrib3dvARB, glVertexAttrib4dvARB}; 
    return (funcs[i]);
  }
  static func getVertexFct(int i)
  {
    static const func funcs[4]={NULL, glVertex2dv, glVertex3dv, glVertex4dv}; 
    return (funcs[i]);
  }
  static func getNormalFct(int i)
  {
    static const func funcs[4]={NULL, NULL, glNormal3dv, NULL}; 
    return (funcs[i]);
  }
  static texfunc getTexCoordnFct(int i)
  {
    static const texfunc funcs[4]={glMultiTexCoord1dvARB, glMultiTexCoord2dvARB, glMultiTexCoord3dvARB, NULL}; 
    return (funcs[i]);
  }
  
};

// -----------------------------------------

template <> struct GLFuncs<unsigned char>
{
public:

  typedef void (APIENTRY *func)(const GLubyte *);

  static func getColor0Fct(int i)
  {
    static const func funcs[4]={NULL, NULL, glColor3ubv, glColor4ubv}; 
    return (funcs[i]);
  }
  
};

// -----------------------------------------
} // namespace GPUmesh
// -----------------------------------------

#endif
