//----------------------------------------------------------
// Generate a cylinder in a mesh
//----------------------------------------------------------
#ifndef __CGenCylinder__
#define __CGenCylinder__
//----------------------------------------------------------
#include "CRenderMesh.h"
//----------------------------------------------------------
#include <list>
#include <static_check.h>
//----------------------------------------------------------
#ifndef M_PI
#  define M_PI	       3.14159265359
#endif
//----------------------------------------------------------
class CGenCylinder
{
private:

  typedef TYPELIST_3(MVF_VERTEX_3,MVF_NORMAL,MVF_TEXCOORD0_2) mvf_cylinder;

public:

  template <class MVF> static void genCylinderStrips(CRenderMesh<MVF>& mesh,int lod)
  {
    CHECK_COMPATIBLE_FORMAT(MVF,CGenCylinder::mvf_cylinder);

    int nbTris=lod*2;

    mesh.begin(RENDERMESH_TRIANGLESTRIP);
    for (int i=0;i<2+nbTris;i++)
    {
      int r=i/2;
      mesh.normal(-sin(M_PI*2.0f*r/(float)(lod)),
                   cos(M_PI*2.0f*r/(float)(lod)),
                   0.0);
      if (i & 1)
      {
        mesh.texcoord0_2(r/(float)(lod-1),
          1.0);
        mesh.vertex_3(cos(M_PI*2.0f*r/(float)(lod)),
          sin(M_PI*2.0f*r/(float)(lod)),
          1.0);
      }
      else
      {
        mesh.texcoord0_2(r/(float)(lod-1),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0f*r/(float)(lod)),
          sin(M_PI*2.0f*r/(float)(lod)),
          0.0);
      }
    }
    mesh.end();
  }

  template <class MVF> static void genCylinderTriangles(CRenderMesh<MVF>& mesh,int lod)
  {
    CHECK_COMPATIBLE_FORMAT(MVF,CGenCylinder::mvf_cylinder);

    int nbTris=lod*2;

    mesh.begin(RENDERMESH_TRIANGLELIST);
    for (int i=0;i<nbTris;i++)
    {
      int r=i/2;

      if (i & 1)
      {
        mesh.texcoord0_2(r/(float)(lod-1),0.0);
        mesh.normal(-sin(M_PI*2.0*r/(float)(lod)),
          cos(M_PI*2.0*r/(float)(lod)),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0*r/(float)(lod)),
          sin(M_PI*2.0*r/(float)(lod)),
          0.0);

        mesh.texcoord0_2((r+1)/(float)(lod-1),0.0);
        mesh.normal(-sin(M_PI*2.0*(r+1)/(float)(lod)),
          cos(M_PI*2.0*(r+1)/(float)(lod)),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0*(r+1)/(float)(lod)),
          sin(M_PI*2.0*(r+1)/(float)(lod)),
          0.0);

        mesh.texcoord0_2(r/(float)(lod-1),1.0);
        mesh.normal(-sin(M_PI*2.0*r/(float)(lod)),
          cos(M_PI*2.0*r/(float)(lod)),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0*r/(float)(lod)),
          sin(M_PI*2.0*r/(float)(lod)),
          1.0);
      }
      else
      {
        mesh.texcoord0_2((r+1)/(float)(lod-1),0.0);
        mesh.normal(-sin(M_PI*2.0*(r+1)/(float)(lod)),
          cos(M_PI*2.0*(r+1)/(float)(lod)),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0*(r+1)/(float)(lod)),
          sin(M_PI*2.0*(r+1)/(float)(lod)),
          0.0);

        mesh.texcoord0_2((r+1)/(float)(lod-1),1.0);
        mesh.normal(-sin(M_PI*2.0*(r+1)/(float)(lod)),
          cos(M_PI*2.0*(r+1)/(float)(lod)),
          1.0);
        mesh.vertex_3(cos(M_PI*2.0*(r+1)/(float)(lod)),
          sin(M_PI*2.0*(r+1)/(float)(lod)),
          1.0);

        mesh.texcoord0_2(r/(float)(lod-1),1.0);
        mesh.normal(-sin(M_PI*2.0*r/(float)(lod)),
          cos(M_PI*2.0*r/(float)(lod)),
          0.0);
        mesh.vertex_3(cos(M_PI*2.0*r/(float)(lod)),
          sin(M_PI*2.0*r/(float)(lod)),
          1.0);
      }
    }
    mesh.end();
  }

};
//----------------------------------------------------------
#endif
//----------------------------------------------------------
