// --------------------------------------------------------------
//
// OpenGL Demo of the RenderMesh class
//
// Dependencies
//
//    * glut
//    * gluX 
//    * libTexture
//    * libGlFont
//
// (c) Sylvain Lefebvre 2004 - http://www.aracknea.net
//
//     Sylvain.Lefebvre@laposte.net
// --------------------------------------------------------------
#pragma warning(disable:4786)
/* -------------------------------------------------------- */
#ifdef WIN32
#include <windows.h>
#else
#include <sys/time.h>
#endif
/* -------------------------------------------------------- */
#include <iostream>
#include <ctime>
#include <cmath>
#include <GL/gl.h>
#include <GL/glut.h>
/* -------------------------------------------------------- */
#include <glux.h>
#include <glfont.h>
/* -------------------------------------------------------- */
#include "GLGPUmesh.h"
/* -------------------------------------------------------- */
#include "CGenSpiral.h"
/* -------------------------------------------------------- */
using namespace GPUmesh;
/* -------------------------------------------------------- */

// define our vertex format

typedef GPUMESH_MVF3(mvf_vertex_3f,mvf_normal_3f,mvf_color0_rgba) mvf_simple;

// associate render meshes and choosen vertex format

typedef GPUmesh_GL_VBO<mvf_simple>         SimpleMesh_VBO;

typedef GPUmesh_GL_DisplayList<mvf_simple> SimpleMesh_DisplayList;

typedef GPUmesh_GL_Calls<mvf_simple>       SimpleMesh_Calls;

/* -------------------------------------------------------- */

// pointers for the three different types of objects

SimpleMesh_VBO         *g_Object_VBO=NULL;
SimpleMesh_DisplayList *g_Object_DisplayList=NULL;
SimpleMesh_Calls       *g_Object_Calls=NULL;

/* -------------------------------------------------------- */

int     g_iMainWindow;        // window glut id
int     g_iWinWidth=640;
int     g_iWinHeight=480;
double  g_dDist=8.0;
double  g_dLightHeight=5.0;
double  g_dLightRadius=3.0;
int     g_iOldX=-1;
int     g_iOldY=-1;
double  g_dRotX=90.0;
double  g_dRotZ=0.0;
bool    g_bStop=false;
bool    g_bWireframe=false;
int     g_iRenderMode=0;
GLfont *g_Font=NULL;

/* -------------------------------------------------------- */

void mainKeyboard(unsigned char key, int x, int y) 
{
  if (key == 'q')
    exit (0);
  else if (key == 'o')
    g_dDist+=1.0;
  else if (key == 'p')
    g_dDist-=1.0;
  else if (key == 'u')
    g_dLightHeight+=0.1;
  else if (key == 'i')
    g_dLightHeight-=0.1;
  else if (key == 't')
    g_dLightRadius+=0.1;
  else if (key == 'y')
    g_dLightRadius-=0.1;
  else if (key == 'f')
    g_bWireframe=!g_bWireframe;
  else if (key == 'n')
    g_iRenderMode=(g_iRenderMode+1) % 3;
}

/* -------------------------------------------------------- */

void mainMouse(int btn, int state, int x, int y) 
{
  if (state == GLUT_DOWN)
  {
    g_iOldX=x;
    g_iOldY=y;
  }
}

/* -------------------------------------------------------- */

void mainMotion(int x,int y)
{
  g_dRotZ+=(g_iOldX-x)*360.0/800.0;
  g_dRotX+=(g_iOldY-y)*360.0/800.0;
  g_iOldX=x;
  g_iOldY=y;
}

/* -------------------------------------------------------- */

void mainReshape(int w,int h)
{
  g_iWinWidth=w;
  g_iWinHeight=h;
  glutPostRedisplay();
}

/* -------------------------------------------------------- */

void mainIdle()
{
  glutPostRedisplay();
}
/* -------------------------------------------------------- */

int getTime()
{
#ifdef WIN32
  return (timeGetTime());
#else
  struct timeval      now;
  unsigned int        ms;

  static struct timeval start;
  static bool           init=false;

  if (!init)
  {
    gettimeofday(&start, NULL);
    init=true;
  }
  gettimeofday(&now, NULL);
  ms=(now.tv_sec-start.tv_sec)*1000+(now.tv_usec-start.tv_usec)/1000;
  return (ms);
#endif
}

/* -------------------------------------------------------- */

void mainRender()
{
  glViewport(0,0,g_iWinWidth,g_iWinHeight);
	
  // clear screen
  glClearColor(0.5,0.5,0.5,0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
  // transform
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(70.0,g_iWinWidth/(double)g_iWinHeight,0.1,1000.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glTranslated(0.0,0.0,-g_dDist);
  glRotated(g_dRotX,1.0,0.0,0.0);
  glRotated(g_dRotZ,0.0,1.0,0.0);

  if (g_bWireframe)
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  else
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

  // draw selected object
  switch (g_iRenderMode)
  {
  case 0:
    g_Object_VBO->render();
    break;
  case 1:
    g_Object_DisplayList->render();
    break;
  case 2:
    g_Object_Calls->render();
    break;
  }

  static char str[256];
  
  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);

  // frame rate
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0,1.0,1.0,0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  static int  frame=0,fps=0;
  static int  last_time=getTime();

  if (frame++ > 10)
  {
    int time=getTime();
    if (time != last_time)
      fps=(frame*1000)/(time-last_time);
    frame=0;
    last_time=time;
  }
  sprintf(str,"%d FPS",fps);
  glColor3d(1.0,1.0,1.0);
  g_Font->printString(0.0,0.0,0.05,str);

  // display render mode
  switch (g_iRenderMode)
  {
  case 0:
    sprintf(str,"vertex buffer object");
    break;
  case 1:
    sprintf(str,"display list");
    break;
  case 2:
    sprintf(str,"function calls");
    break;
  }
  glColor3d(1.0,1.0,1.0);
  g_Font->printString(0.0,0.95,0.05,str);
  glPopAttrib();

  // swap buffers
  glutSwapBuffers();
}
/* -------------------------------------------------------- */
int main(int argc, char **argv) 
{
  srand(time(NULL));
  // GLUT
  glutInit(&argc, argv);
  glutInitWindowSize(g_iWinWidth,g_iWinHeight);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  // main window
  g_iMainWindow=glutCreateWindow("Render mesh OpenGL exemple");
  glutMouseFunc(mainMouse);
  glutMotionFunc(mainMotion);
  glutKeyboardFunc(mainKeyboard);
  glutDisplayFunc(mainRender);
  glutReshapeFunc(mainReshape);
  glutSetWindow(g_iMainWindow);
  glutIdleFunc(mainIdle);

  // gluX
  gluxInit();

  // font
  g_Font=new GLfont("default.tga");

  // create objects

  g_Object_VBO         = new SimpleMesh_VBO();
  g_Object_DisplayList = new SimpleMesh_DisplayList();
  g_Object_Calls       = new SimpleMesh_Calls();

  // create geometry
  //   the geomtry producer (CGenSpiral) can be used with the three 
  //   types of meshes (and is also compatible with DirectX - see 
  //   rendermesh_d3d).
  GenSpiral<SimpleMesh_VBO>        ::genSpiral(*g_Object_VBO        ,50);
  GenSpiral<SimpleMesh_DisplayList>::genSpiral(*g_Object_DisplayList,50);
  GenSpiral<SimpleMesh_Calls>      ::genSpiral(*g_Object_Calls      ,50);

  // help
  printf("[q]     - quit\n");
  printf("[o]     - zoom out\n");
  printf("[p]     - zoom in\n");
  printf("[n]     - next rendering method\n");
  printf("[f]     - wireframe\n");
	
  // gl init
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);

  // setup light
  float spec[]={1.0,1.0,1.0,1.0};
  glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,spec);
  float shin[]={20.0,20.0,20.0,20.0};
  glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,shin);
  float f=1.0;
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER,&f);

  // let's go
  glutMainLoop();

  return (0);
}
/* -------------------------------------------------------- */
